//  Demo 20: ListAnimationThreadA.java
//
//   A Thread to Animate a Linked List of MoveableSquare Objects.
//   Uses the MoveableSquare and SimpleObjectList classes.   
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999

// This continues the single animation Thread approach to animating
// groups of objects.  We shall repeatedly traverse a SimpleObject
// List and move and redraw each of the MoveableSquares it contains.
      
import java.awt.*;
import MoveableSquare;
import SimpleObjectList;

// A ListAnimationThreadA object should be
//
//   1. A Thread so it can run as a separate Thread of execution.
//
// A ListAnimationThreadA object should know
//
//   1. The SimpleObjectList contain MoveableSqaures which it is
//        going to animate (state variable).
//   2. Its parent object, eg. the Frame or Applet Panel on which 
//        the animation will be displayed (state variable).
//   3. The Graphics and Image buffer objects it is going to use
//        for background drawing (state variables).
//   4. Whether it should keep running or not (in case we want to
//      suspend and then restart the animation) (state variable).
//   5. How fast to run the animation (state variable).
//   6. How to set and return the SimpleObjectList object (methods).
//   7. How to set and return the parent object (methods).
//   8. How to set and return the Image buffer object (methods).
//   9. How to set and return the keep running variable (methods).
//  10. How to set and return the pause time variable to control 
//        the animation speed (methods).

public class ListAnimationThreadA extends Thread {

   private SimpleObjectList sobl;  // A linked list of objects
                                   // which will contain Moveable
                                   // Square objects.
 

   private Component parent;  // the Canvas, Frame or Applet Panel
                              // on which we draw the animation

   private Graphics g;  // our parent's Graphics object, on which 
                        // we draw

   private Graphics gBuffer;   // For double buffering, as usual.
   private Image imageBuffer;  // As in the MovingSquareThread 
                               // class, we shall pass them in via
                               // the Constructor, just in case
                               // we might ever need more than one
                               // ListAnimationThreadA object.

   private boolean keepRunning; // should we keep running?
   private int pauseTime;       // in milliseconds.  The larger 
                                // this is, the slower we run.

   // Constructor: as usual, just initializes variables.  The
   // parameters are:
     //  Square sobl      : the SimpleObjectList of MoveableSquares
     //                     to be animated.
     //  Component parent : the Canvas, Frame or Applet Panel
     //                     on which we draw the animation
     //  Image imageBuffer : the Image buffer object we use for
     //                      background drawing
     //  int pauseTime    : the pauseTime in milliseconds
     //  We shall initialize keepRunning and g in the init() method.
   public ListAnimationThreadA(SimpleObjectList sobl, 
                               Component parent, 
                               Image imageBuffer,
                               int pauseTime) {

       this.sobl = sobl;
       this.parent = parent;

       // we can initialize the imageBuffer and gBuffer variables
       // here in the constructor, rather than doing this in init(),
       // because THEY already exist, even if we don't.
       this.imageBuffer = imageBuffer;
       this.gBuffer = imageBuffer.getGraphics();

       this.pauseTime = pauseTime;

    } // Constructor ListAnimationThreadA()

   // init(): initializes keepRunning and the Graphics object we
   //         draw on.  Remember, we can't initialize the Graphics
   //         object in our Constructor, because we actually have
   //         to exist before a call to parent.getGraphics() will
   //         work.
   //         This is another example of an "internal method".  It
   //         will be called from run().  Therefore, it is declared 
   //         to be private.
   private void init() {

     keepRunning = true;

     // initialize g to the Graphics object (context) of our parent.
     // g is now the g of our parent's paint(Graphics g) method.
     g = parent.getGraphics();

     // no need to call repaint() because we are not going to
     // do our drawing in paint()!

   } // init()

   // set our SimpleObjectList object
   public void setSimpleObjectList(SimpleObjectList sobl){

      this.sobl = sobl;

   } // setSimpleObjectList()

   // return our SimpleObjectList object
   public SimpleObjectList getSimpleObjectList() {

      return sobl;

   } // getSimpleObjectList()

   // set our parent object
   public void setParent(Component parent) {

      this.parent = parent;

   } // setParent()

   // return our Parent object
   public Component getParent() {

      return parent;

   } // getParent()

   // set our Image buffer object
   public void setImageBuffer(Image imageBuffer) {

     this.imageBuffer = imageBuffer;

   } // setImageBuffer()

   // return our Image buffer object
   public Image getImageBuffer(Image imageBuffer) {

      return imageBuffer;

   } // getImageBuffer()

   // set our pauseTime
   public void setPauseTime(int pauseTime) {

      this.pauseTime = pauseTime;

   } // setPauseTime()

   // return our pauseTime
   public int getPauseTime(int pauseTime) {

      return pauseTime;

   } // getPauseTime()

   // Here is where we do our assigned task, animate the Simple
   // ObjectList containing MoveableSquare Objects.  All Threads
   // have to have a run method.
   public void run() {

      SimpleObjectList p;  // classic pointer to current linked
                           // list node

      MoveableSquare ms;   // MoveableSquare in linked list node
                           // pointed to by p

      Color bg = parent.getBackground();  // background color

      init();  // initialize Graphics and Image buffers

      // This is interesting.  For the first time, we are going to
      // do our drawing outside of the paint() method.  This is so
      // we can continually draw in one Thread of execution while
      // our creator application (a Frame or an Applet) does other
      // things like event processing.  Also, we don't have a 
      // paint() method because we are not a Frame or Applet Panel.

      // Here comes the animation, and it runs forever!
      while(keepRunning) {

         // classic linked list traversal 
         p = sobl;
         while (p != null) {

            ms = (MoveableSquare) p.ob;
         
            // move ms -- do drawing offscreen on the Graphics
            //         buffer object
            if (ms.getDT() <= ms.getD()) {
               ms.draw(gBuffer, bg);  // hide square
               ms.move(ms.getRX(), ms.getRY());
               ms.draw(gBuffer);
               ms.incrDT(1);  // increment distance traveled
            }

            p = p.next;

         }

         //draw completed image on the screen
         g.drawImage(imageBuffer, 0, 0, parent);

         // To pause, so the animation doesn't go too fast, use
         // the sleep() method of the Thread class.  Unfortunately,
         // it throws an InterruptedException, so we have to embed
         // our simple pause statement in a try -- catch clause.
         if (pauseTime >= 1) {
            try {
              Thread.sleep(pauseTime);  // pauseTime ms pause
            }
            catch (InterruptedException e) {
              // do nothing
            }
         } // if

      } // while

   } // run()

} // class ListAnimationThreadA